import 'package:flutter/widgets.dart';

Map<String, Function> _map = Map<String, Function>();

class FunctionRegistry {
  static void register<T>(String name, Function function) {
    debugPrint('注册Function name=$name');
    assert(name.startsWith("@"), "Function name 需要以 @ 开头 ");
    assert(!_map.containsKey(name), "Function name 已存在");
    assert(_checkType<T>(name, function));
    _map[name] = function;
  }

  static T call<T>(String name, {dynamic data}) {
    debugPrint('call Function name=$name');
    assert(name.startsWith("@"), "Function name 需要以 @ 开头 ");
    if (!_map.containsKey(name)) throw FormatException('Function:$name 未注册');

    assert(_checkType<T>(name, _map[name]!, data: data, isCall: true));
    return _map[name]!.call(data);
  }
}

bool _checkType<T>(String name, Function function,
    {bool isCall = false, dynamic data}) {
  var tType = T.toString();
  var funReturnType = function.runtimeType.toString().split("=>").last.trim();
  var funDataType = function.runtimeType.toString().split("=>").first.trim();
  var dataType = data.runtimeType.toString();
  debugPrint(
      "tType=$tType funReturnType=$funReturnType tType=funReturnType？${tType == funReturnType}   ${isCall ? 'funDataType=$funDataType  dataType=$dataType  ${(funDataType == "(dynamic)" && dataType == "Null")}' : ''} ");
  assert(
      tType == funReturnType || (tType == "dynamic" && funReturnType == "Null"),
      "类型$name错误，期望返回类型=$tType 然而Function实际返回类型=${funReturnType.split("=>").last.trim()}");
  if (isCall)
    assert(
        funDataType.contains(dataType) ||
            (funDataType == "(dynamic)" && dataType == "Null"),
        "data类型错误，Function 【$name】 期望入参data类型=$funDataType实际入参类型=$dataType");
  else {
    assert(_map.containsKey(name), "Function name：$name 已被占用");
  }
  return true;
}
